@@ -33,8 +33,7 @@ struct _EvLinkAccessiblePrivate {
 
         EvHyperlink      *hyperlink;
 
-        gint        start_index;
-        gint        end_index;
+        gchar      *name;
 };
 
 struct _EvHyperlink {
@@ -106,8 +105,6 @@ ev_hyperlink_get_start_index (AtkHyperlink *atk_hyperlink)
                 return -1;
 
         impl_priv = hyperlink->link_impl->priv;
-        if (impl_priv->start_index != -1)
-		return impl_priv->start_index;
 
 	view = ev_page_accessible_get_view (impl_priv->page);
         if (!view->page_cache)
@@ -126,10 +123,8 @@ ev_hyperlink_get_start_index (AtkHyperlink *atk_hyperlink)
                 c_x = rect->x1 + (rect->x2 - rect->x1) / 2.;
                 c_y = rect->y1 + (rect->y2 - rect->y1) / 2.;
                 if (c_x >= impl_priv->area.x1 && c_x <= impl_priv->area.x2 &&
-                    c_y >= impl_priv->area.y1 && c_y <= impl_priv->area.y2) {
-                        impl_priv->start_index = i;
+                    c_y >= impl_priv->area.y1 && c_y <= impl_priv->area.y2)
                         return i;
-		}
         }
 
         return -1;
@@ -144,18 +139,11 @@ ev_hyperlink_get_end_index (AtkHyperlink *atk_hyperlink)
         EvRectangle             *areas = NULL;
         guint                    n_areas = 0;
         guint                    i;
-        gint                     start_index;
 
         if (!hyperlink->link_impl)
                 return -1;
 
         impl_priv = hyperlink->link_impl->priv;
-        if (impl_priv->end_index != -1)
-		return impl_priv->end_index;
-
-        start_index = ev_hyperlink_get_start_index (atk_hyperlink);
-        if (start_index == -1)
-		return -1;
 
 	view = ev_page_accessible_get_view (impl_priv->page);
         if (!view->page_cache)
@@ -167,17 +155,15 @@ ev_hyperlink_get_end_index (AtkHyperlink *atk_hyperlink)
         if (!areas)
                 return -1;
 
-        for (i = start_index + 1; i < n_areas; i++) {
+        for (i = n_areas - 1; i >= 0; i--) {
                 EvRectangle *rect = areas + i;
                 gdouble      c_x, c_y;
 
                 c_x = rect->x1 + (rect->x2 - rect->x1) / 2.;
                 c_y = rect->y1 + (rect->y2 - rect->y1) / 2.;
-                if (c_x < impl_priv->area.x1 || c_x > impl_priv->area.x2 ||
-                    c_y < impl_priv->area.y1 || c_y > impl_priv->area.y2) {
-                        impl_priv->end_index = i;
-                        return i;
-		}
+                if (c_x >= impl_priv->area.x1 && c_x <= impl_priv->area.x2 &&
+                    c_y >= impl_priv->area.y1 && c_y <= impl_priv->area.y2)
+                        return i + 1;
         }
 
         return -1;
@@ -203,10 +189,71 @@ ev_hyperlink_init (EvHyperlink *link)
 
 static void ev_link_accessible_hyperlink_impl_iface_init (AtkHyperlinkImplIface *iface);
 static void ev_link_accessible_action_interface_init     (AtkActionIface        *iface);
+static void ev_link_accessible_component_iface_init      (AtkComponentIface     *iface);
 
 G_DEFINE_TYPE_WITH_CODE (EvLinkAccessible, ev_link_accessible, ATK_TYPE_OBJECT,
 			 G_IMPLEMENT_INTERFACE (ATK_TYPE_HYPERLINK_IMPL, ev_link_accessible_hyperlink_impl_iface_init)
-			 G_IMPLEMENT_INTERFACE (ATK_TYPE_ACTION, ev_link_accessible_action_interface_init))
+			 G_IMPLEMENT_INTERFACE (ATK_TYPE_ACTION, ev_link_accessible_action_interface_init)
+			 G_IMPLEMENT_INTERFACE (ATK_TYPE_COMPONENT, ev_link_accessible_component_iface_init))
+
+static const gchar *
+ev_link_accessible_get_name (AtkObject *atk_object)
+{
+	EvLinkAccessiblePrivate *priv;
+	gint start_index;
+	gint end_index;
+
+	priv = EV_LINK_ACCESSIBLE (atk_object)->priv;
+	if (priv->name)
+		return priv->name;
+
+	start_index = ev_hyperlink_get_start_index (ATK_HYPERLINK (priv->hyperlink));
+	end_index = ev_hyperlink_get_end_index (ATK_HYPERLINK (priv->hyperlink));
+	priv->name = atk_text_get_text (ATK_TEXT (atk_object_get_parent (atk_object)), start_index, end_index);
+
+	return priv->name;
+}
+
+static AtkObject *
+ev_link_accessible_get_parent (AtkObject *atk_object)
+{
+	EvLinkAccessiblePrivate *priv = EV_LINK_ACCESSIBLE (atk_object)->priv;
+
+	return ATK_OBJECT (priv->page);
+}
+
+static AtkStateSet *
+ev_link_accessible_ref_state_set (AtkObject *atk_object)
+{
+	AtkStateSet *state_set;
+	AtkStateSet *copy_set;
+	AtkStateSet *page_accessible_state_set;
+	EvLinkAccessible *self;
+	EvViewAccessible *view_accessible;
+	EvView *view;
+	gint page;
+
+	self = EV_LINK_ACCESSIBLE (atk_object);
+	state_set = ATK_OBJECT_CLASS (ev_link_accessible_parent_class)->ref_state_set (atk_object);
+	atk_state_set_clear_states (state_set);
+
+	page_accessible_state_set = atk_object_ref_state_set (ATK_OBJECT (self->priv->page));
+	copy_set = atk_state_set_or_sets (state_set, page_accessible_state_set);
+
+	view_accessible = ev_page_accessible_get_view_accessible (self->priv->page);
+	page = ev_page_accessible_get_page (self->priv->page);
+	if (!ev_view_accessible_is_doc_rect_showing (view_accessible, page, &self->priv->area))
+		atk_state_set_remove_state (copy_set, ATK_STATE_SHOWING);
+
+	view = ev_page_accessible_get_view (self->priv->page);
+	if (!view->focused_element || view->focused_element->data != self->priv->link)
+		atk_state_set_remove_state (copy_set, ATK_STATE_FOCUSED);
+
+	g_object_unref (state_set);
+	g_object_unref (page_accessible_state_set);
+
+	return copy_set;
+}
 
 static void
 ev_link_accessible_finalize (GObject *object)
@@ -214,6 +261,7 @@ ev_link_accessible_finalize (GObject *object)
         EvLinkAccessible *link = EV_LINK_ACCESSIBLE (object);
 
         g_clear_object (&link->priv->hyperlink);
+        g_free (link->priv->name);
 
         G_OBJECT_CLASS (ev_link_accessible_parent_class)->finalize (object);
 }
@@ -222,18 +270,22 @@ static void
 ev_link_accessible_class_init (EvLinkAccessibleClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
+        AtkObjectClass *atk_class = ATK_OBJECT_CLASS (klass);
 
         object_class->finalize = ev_link_accessible_finalize;
 
         g_type_class_add_private (klass, sizeof (EvLinkAccessiblePrivate));
+
+        atk_class->get_parent = ev_link_accessible_get_parent;
+        atk_class->get_name = ev_link_accessible_get_name;
+        atk_class->ref_state_set = ev_link_accessible_ref_state_set;
 }
 
 static void
 ev_link_accessible_init (EvLinkAccessible *link)
 {
+        atk_object_set_role (ATK_OBJECT (link), ATK_ROLE_LINK);
         link->priv = G_TYPE_INSTANCE_GET_PRIVATE (link, EV_TYPE_LINK_ACCESSIBLE, EvLinkAccessiblePrivate);
-        link->priv->start_index = -1;
-        link->priv->end_index = -1;
 }
 
 static AtkHyperlink *
@@ -304,6 +356,55 @@ ev_link_accessible_action_interface_init (AtkActionIface *iface)
 	iface->get_name = ev_link_accessible_action_get_name;
 }
 
+static void
+ev_link_accessible_get_extents (AtkComponent *atk_component,
+				gint         *x,
+				gint         *y,
+				gint         *width,
+				gint         *height,
+				AtkCoordType coord_type)
+{
+	EvLinkAccessible *self;
+	EvViewAccessible *view_accessible;
+	gint page;
+	EvRectangle atk_rect;
+
+	self = EV_LINK_ACCESSIBLE (atk_component);
+	view_accessible = ev_page_accessible_get_view_accessible (self->priv->page);
+	page = ev_page_accessible_get_page (self->priv->page);
+	_transform_doc_rect_to_atk_rect (view_accessible, page, &self->priv->area, &atk_rect, coord_type);
+	*x = atk_rect.x1;
+	*y = atk_rect.y1;
+	*width = atk_rect.x2 - atk_rect.x1;
+	*height = atk_rect.y2 - atk_rect.y1;
+}
+
+static gboolean
+ev_link_accessible_grab_focus (AtkComponent *atk_component)
+{
+	EvLinkAccessible *self;
+	EvView *view;
+	EvMappingList *link_mapping;
+	EvMapping *mapping;
+	gint page;
+
+	self = EV_LINK_ACCESSIBLE (atk_component);
+	view = ev_page_accessible_get_view (self->priv->page);
+	page = ev_page_accessible_get_page (self->priv->page);
+	link_mapping = ev_page_cache_get_link_mapping (view->page_cache, page);
+	mapping = ev_mapping_list_find (link_mapping, self->priv->link);
+	_ev_view_set_focused_element (view, mapping, page);
+
+	return TRUE;
+}
+
+static void
+ev_link_accessible_component_iface_init (AtkComponentIface *iface)
+{
+	iface->get_extents = ev_link_accessible_get_extents;
+	iface->grab_focus = ev_link_accessible_grab_focus;
+}
+
 EvLinkAccessible *
 ev_link_accessible_new (EvPageAccessible *page,
                         EvLink           *link,
