@@ -937,16 +937,125 @@ app_help_cb (GSimpleAction *action,
         ev_application_show_help (application, NULL, NULL);
 }
 
+static void
+app_about_cb (GSimpleAction *action,
+              GVariant      *parameter,
+              gpointer       user_data)
+{
+        const char *authors[] = {
+                "Martin Kretzschmar <m_kretzschmar@gmx.net>",
+                "Jonathan Blandford <jrb@gnome.org>",
+                "Marco Pesenti Gritti <marco@gnome.org>",
+                "Nickolay V. Shmyrev <nshmyrev@yandex.ru>",
+                "Bryan Clark <clarkbw@gnome.org>",
+                "Carlos Garcia Campos <carlosgc@gnome.org>",
+                "Wouter Bolsterlee <wbolster@gnome.org>",
+                "Christian Persch <chpe" "\100" "gnome.org>",
+                NULL
+        };
+        const char *documenters[] = {
+                "Nickolay V. Shmyrev <nshmyrev@yandex.ru>",
+                "Phil Bull <philbull@gmail.com>",
+                "Tiffany Antpolski <tiffany.antopolski@gmail.com>",
+                NULL
+        };
+        const char *license[] = {
+                N_("Evince is free software; you can redistribute it and/or modify "
+                   "it under the terms of the GNU General Public License as published by "
+                   "the Free Software Foundation; either version 2 of the License, or "
+                   "(at your option) any later version.\n"),
+                N_("Evince is distributed in the hope that it will be useful, "
+                   "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+                   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
+                   "GNU General Public License for more details.\n"),
+                N_("You should have received a copy of the GNU General Public License "
+                   "along with Evince; if not, write to the Free Software Foundation, Inc., "
+                   "51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA\n")
+        };
+        char *license_trans;
+#ifdef ENABLE_NLS
+        const char **p;
+
+        for (p = authors; *p; ++p)
+                *p = _(*p);
+
+        for (p = documenters; *p; ++p)
+                *p = _(*p);
+#endif
+
+        license_trans = g_strconcat (_(license[0]), "\n",
+                                     _(license[1]), "\n",
+                                     _(license[2]), "\n",
+                                     NULL);
+
+        gtk_show_about_dialog (NULL,
+                               "name", _("Evince"),
+                               "version", VERSION,
+                               "copyright", _("© 1996–2014 The Evince authors"),
+                               "license", license_trans,
+                               "website", "https://wiki.gnome.org/Apps/Evince",
+                               "comments", _("Document Viewer"),
+                               "authors", authors,
+                               "documenters", documenters,
+                               "translator-credits", _("translator-credits"),
+                               "logo-icon-name", "evince",
+                               "wrap-license", TRUE,
+                               NULL);
+
+        g_free (license_trans);
+}
+
 static void
 ev_application_startup (GApplication *gapplication)
 {
         const GActionEntry app_menu_actions[] = {
                 { "help", app_help_cb, NULL, NULL, NULL },
+                { "about", app_about_cb, NULL, NULL, NULL }
+        };
+
+        const gchar *action_accels[] = {
+          "win.open",                   "<Ctrl>O", NULL,
+          "win.open-copy",              "<Ctrl>N", NULL,
+          "win.save-copy",              "<Ctrl>S", NULL,
+          "win.print",                  "<Ctrl>P", NULL,
+          "win.copy",                   "<Ctrl>C", "<Ctrl>Insert", NULL,
+          "win.select-all",             "<Ctrl>A", NULL,
+          "win.save-settings",          "<Ctrl>T", NULL,
+          "win.go-first-page",          "<Ctrl>Home", NULL,
+          "win.go-last-page",           "<Ctrl>End", NULL,
+          "win.add-bookmark",           "<Ctrl>D", NULL,
+          "win.close",                  "<Ctrl>W", NULL,
+          "win.escape",                 "Escape", NULL,
+          "win.find",                   "<Ctrl>F", "slash", NULL,
+          "win.find-next",              "<Ctrl>G", NULL,
+          "win.find-previous",          "<Ctrl><Shift>G", NULL,
+          "win.select-page",            "<Ctrl>L", NULL,
+          "win.go-backward",            "<Shift>Page_Up", NULL,
+          "win.go-forward",             "<Shift>Page_Down", NULL,
+          "win.go-next-page",           "n", NULL,
+          "win.go-previous-page",       "p", NULL,
+          "win.go-back-history",        "<alt>P", NULL,
+          "win.go-forward-history",     "<alt>N", NULL,
+          "win.sizing-mode::fit-page",  "f", NULL,
+          "win.sizing-mode::fit-width", "w", NULL,
+          "win.open-menu",              "F10", NULL,
+          "win.caret-navigation",       "F7", NULL,
+          "win.zoom-in",                "plus", "<Ctrl>plus", "KP_Add", "<Ctrl>KP_Add", NULL,
+          "win.zoom-out",               "minus", "<Ctrl>minus", "KP_Subtract", "<Ctrl>KP_Subtract", NULL,
+          "win.show-side-pane",         "F9", NULL,
+          "win.fullscreen",             "F11", NULL,
+          "win.presentation",           "F5", NULL,
+          "win.rotate-left",            "<Ctrl>Left", NULL,
+          "win.rotate-right",           "<Ctrl>Right", NULL,
+          "win.inverted-colors",        "<Ctrl>I", NULL,
+          "win.reload",                 "<Ctrl>R", NULL,
+          NULL
         };
 
         EvApplication *application = EV_APPLICATION (gapplication);
         GtkBuilder *builder;
         GError *error = NULL;
+        const gchar **it;
 
         G_APPLICATION_CLASS (ev_application_parent_class)->startup (gapplication);
 
@@ -955,12 +1064,15 @@ ev_application_startup (GApplication *gapplication)
                                          application);
 
         builder = gtk_builder_new ();
-        gtk_builder_add_from_resource (builder, "/org/gnome/evince/shell/ui/appmenu.ui", &error);
+        gtk_builder_add_from_resource (builder, "/org/gnome/evince/shell/ui/menus.ui", &error);
         g_assert_no_error (error);
 
         gtk_application_set_app_menu (GTK_APPLICATION (application),
                                       G_MENU_MODEL (gtk_builder_get_object (builder, "appmenu")));
         g_object_unref (builder);
+
+        for (it = action_accels; it[0]; it += g_strv_length ((gchar **)it) + 1)
+                gtk_application_set_accels_for_action (GTK_APPLICATION (application), it[0], &it[1]);
 }
 
 static void
